/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: TransferSrmCopyStat structure
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferSrmCopyStat.h,v 1.3 2007/02/21 21:44:10 badino Exp $
 *  Release: $Name: glite-data-transfer-url-copy_R_2_1_1_1 $
 */
#ifndef GLITE_DATA_TRANSFER_URL_COPY_TRANSFERSRMCOPYSTAT_H_
#define GLITE_DATA_TRANSFER_URL_COPY_TRANSFERSRMCOPYSTAT_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/urlcopy/UrlCopyError.h"
#include "glite/data/transfer/urlcopy/UrlCopyStatus.h"

#include <vector>

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {
/*
 * forward declaration
 */
struct SrmCopyStat;

/**
 * Transfer UrlCopy File Request Structure
 */	
struct TransferSrmCopyStat {
    
    typedef glite::data::agents::longlong longlong;
    
    /**
     * SrmCopy File Request Stat Structure
     */	
    struct FileRequest {
        std::string  src_name;
        std::string  dest_name;
        bool         exists;
        longlong     size;
        // Status
        UrlCopyStatus::Value status;
        // Error
        UrlCopyError error;
        // Timestamps
        time_t       start_time;
        time_t       done_time;
        
        FileRequest();
    };

    UrlCopyStatus::Value status;
    // The overall transfer error
    UrlCopyError         error;
    // File Requests
    std::vector<FileRequest> files;
    // Request token
    std::string          token;
    // Space token
    std::string          destination_space_token;
    // User Description tag
    std::string          user_tag;
    // Transfer request info
    time_t               start_time;
    time_t               finish_time;
    time_t               last_refresh_time;
    time_t               src_prep_start_time;
    time_t               src_prep_done_time;
    time_t               dest_prep_start_time;
    time_t               dest_prep_done_time;
    time_t               copy_start_time;
    time_t               copy_done_time;
    time_t               final_start_time;
    time_t               final_done_time;
   
    /**
     * Default Constructor
     */	
    TransferSrmCopyStat();
    /**
     * Constructor
     */	
    TransferSrmCopyStat(const struct SrmCopyStat * const stat);
};

} // end namespace urlcopy
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_TRANSFERSRMCOPYSTAT_H_
