/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: TransferUrlCopyStat structure
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TransferUrlCopyStat.h,v 1.5 2008/01/25 13:21:13 ptedesco Exp $
 *  Release: $Name: glite-data-transfer-url-copy_R_2_1_1_1 $
 */
#ifndef GLITE_DATA_TRANSFER_URL_COPY_TRANSFERURLCOPYSTAT_H_
#define GLITE_DATA_TRANSFER_URL_COPY_TRANSFERURLCOPYSTAT_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/urlcopy/UrlCopyError.h"
#include "glite/data/transfer/urlcopy/UrlCopyStatus.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {
/*
 * forward declaration
 */
struct UrlCopyStat;
    
/**
 * Transfer UrlCopy File Request Structure
 */ 
struct TransferUrlCopyStat {
    
    typedef glite::data::agents::longlong longlong;
    
    /**
     * UrlCopy File Request Stat Structure
     */ 
    struct FileRequest {
        std::string  srm_endpoint;
        std::string  srm_version;
        std::string  name;
        std::string  tx_name;
        std::string  token;
        bool         exists;
        longlong     size;
        // Error
        UrlCopyError error;
        // Time info
        time_t       prep_start_time;
        time_t       prep_done_time;
        time_t       final_start_time;
        time_t       final_done_time;
        
        FileRequest();
    };

    /**
     * UrlCopy File Transfer Stat Structure
     */ 
    struct FileTransfer {
        longlong     size;
        // Error
        UrlCopyError error;
        // Transfer info
        time_t       start_time;
        time_t       done_time;
        time_t       first_marker;
        time_t       last_marker;
        longlong     transferred_bytes;
        float        avg_throughput;
        float        instant_throughput;
        float        percent_done;
        int          markers;    
        
        FileTransfer();
    };

    UrlCopyStatus::Value status;
    // The overall transfer error
    UrlCopyError         error;
    // Source File
    FileRequest          source;
    // Destination File
    FileRequest          destination;
    std::string          destination_space_token;
    // User Description tag
    std::string          user_tag;
    // Transfer 
    FileTransfer         tx;
    // Transfer request info
    time_t               start_time;
    time_t               finish_time;
    time_t               last_refresh_time;
    
    /**
     * Default Constructor
     */ 
    TransferUrlCopyStat();
    /**
     * Constructor
     */ 
    TransferUrlCopyStat(const struct UrlCopyStat * const stat);
};

} // end namespace urlcopy
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_TRANSFERURLCOPYSTAT_H_
