/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: Transfer Base class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferBase.h,v 1.4 2008/02/20 10:59:22 ptedesco Exp $
 *  Release: $Name: glite-data-transfer-url-copy_R_2_1_1_1 $
 */

#ifndef GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERBASE_H_
#define GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERBASE_H_

#include <string>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {
namespace details {

/**
 * The Base class for Transfer Url Copy Service Classes
 * @stereotype abstract
 */
class TransferBase : boost::noncopyable {
public:

    /**
     * The Transfer Url Copy Arguments
     */
    struct Args{
        // Configuration
        bool        overwrite;
        bool        verbose;
        bool        console_logging;
        // destination space token
        std::string dest_space_token;
        std::string dest_token_descr;
        // User's Description Tag
        std::string user_tag;
        // Timeouts
        int         http_timeout;
        
        // added in 2_1_0_1
        //----------------------------------------------------------------------
        // source space token
        std::string source_space_token;
        std::string source_token_descr;
        // flag to enable/disable checks for existance of target directory
        bool        enable_dir_check;
        // flag to enable/disable ping
        bool        enable_ping_source;
        bool        enable_ping_dest;
        
        /**
         * Constructor
         */
        Args():
            overwrite(false),
            verbose(false),
            console_logging(false),
            http_timeout(-1),
            enable_dir_check(true),
            enable_ping_source(true),
            enable_ping_dest(true){}
    };

    /**
     * Destructor
     */
    virtual ~TransferBase(){}

    /**
     * Return the transfer id
     */
    const std::string& id() const {return m_id;}

    /**
     * Return the exec name used
     */
    virtual const std::string& execName() const throw() = 0;

    /**
     * Execute the transfer in current the process
     */
    virtual void exec(bool enable_console = false) /*throw(RuntimeError)*/ = 0;

protected:
    /**
     * Constructor
     */
    explicit TransferBase(const std::string& id):m_id(id){}
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    TransferBase();

protected:
    /**
     * The transfer id
     */
    std::string m_id;
};

} // namespace details
} // namespace urlcopy
} // namespace transfer
} // namespace data
} // namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERBASE_H_
