/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: TransferService TemplateClass
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferService.h,v 1.2 2007/01/31 09:40:27 badino Exp $
 *  Release: $Name: glite-data-transfer-url-copy_R_2_1_1_1 $
 */

#ifndef GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSERVICE_H_

#include "glite/data/transfer/urlcopy/details/TransferBase.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {
namespace details {

/**
 * The Base class for Transfer Url Copy Service Classes
 * @stereotype abstract
 */
template<typename S>
class TransferService : public TransferBase {
public:

    /**
     * Destructor
     */
    virtual ~TransferService(){}

    /**
     * Retrieve transfer status
     * @param check [IN] perform a check that the transfer process service the 
     *    request is still active. 
     */
    S status(bool check = true) /*throw(RuntimeError)*/;

    /**
     * Stop the transfer 
     */
    virtual void stop() /*throw (RuntimeError)*/ = 0;

    /**
     * Abort transfer
     */
    virtual void abort() /* throw() */  = 0;

    /**
     * Clean the transfer log files
     */
    virtual void clean() /* throw(LogicError) */  = 0;

protected:
    
    /**
     * Constructor
     */
    explicit TransferService(const std::string& id):TransferBase(id){}
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    TransferService();
};

} // namespace details
} // namespace urlcopy
} // namespace transfer
} // namespace data
} // namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSERVICE_H_
